@@ -3,6 +3,9 @@
 #ifndef _aring_CC_hpp_
 #define _aring_CC_hpp_
 
+#include "interface/gmp-util.h"  // for moveTo_gmpCC
+#include "interface/random.h"    // for randomDouble
+
 #include "aring.hpp"
 #include "buffer.hpp"
 #include "ringelem.hpp"
@@ -17,7 +20,7 @@ namespace M2 {
 /**
 \ingroup rings
 */
-class ARingCC : public RingInterface
+class ARingCC : public SimpleARing<ARingCC>
 {
   // approximate real numbers, implemented as doubles.
 
@@ -89,6 +92,11 @@ class ARingCC : public RingInterface
     result = * a.get_cc_doubles();
   }
 
+  const ElementType& from_ring_elem_const(const ring_elem& a) const
+  {
+    return *a.get_cc_doubles();
+  }
+
   // 'init', 'init_set' functions
 
   void init(ElementType& result) const
@@ -105,7 +113,7 @@ class ARingCC : public RingInterface
     result.im = 0.0;
   }
 
-  void clear(ElementType& result) const
+  static void clear(ElementType& result)
   {
     // do nothing
   }
