@@ -66,6 +66,15 @@ class CoefficientRingZZp : public M2::SimpleARing<CoefficientRingZZp>
     if (a < 0) a += p;
     result = log_table[a];
   }
+  
+  void set_from_mpz(elem &result, mpz_t a) const
+  {
+    mpz_t tmp;
+    mpz_init_set_si(tmp, p);        // Convert int p to mpz_t
+    mpz_mod(a, a, tmp);             // a = a mod p (always non-negative)
+    mpz_clear(tmp);
+    result = log_table[mpz_get_si(a)];
+  }
 
   long coerceToLongInteger(const elem &f) const
   {
@@ -226,6 +235,7 @@ class CoefficientRingR
   void set_zero(elem &result) const { result = R->zero(); }
   void set(elem &result, elem a) const { result = a; }
   void set_from_long(elem &result, long a) const { result = R->from_long(a); }
+  void set_from_mpz(elem &result, mpz_t a) const { result = R->from_int(a); }
   bool is_zero(elem result) const { return R->is_zero(result); }
   bool is_equal(elem a, elem b) const { return R->is_equal(a, b); }
   bool is_unit(elem f) const { return R->is_unit(f); }
