@@ -55,31 +55,9 @@ void F4toM2Interface::from_M2_vec(const VectorArithmetic* VA,
   delete [] lexp;
 }
 
-void F4toM2Interface::poly_set_degrees(const VectorArithmetic* VA,
-                                       const MonomialInfo *MI,
-                                       const M2_arrayint wts,
-                                       const GBF4Polynomial &f,
-                                       int &deg_result,
-                                       int &alpha)
-{
-  const monomial_word *w = f.monoms;
-  monomial_word leaddeg = MI->monomial_weight(w, wts);
-  monomial_word deg = leaddeg;
-
-  for (int i = 1; i < f.len; i++)
-    {
-      w = w + MI->monomial_size(w);
-      monomial_word degi = MI->monomial_weight(w, wts);
-      if (degi > deg) deg = degi;
-    }
-  alpha = static_cast<int>(deg - leaddeg);
-  deg_result = static_cast<int>(deg);
-}
-
 void F4toM2Interface::from_M2_matrix(const VectorArithmetic* VA,
                                      const MonomialInfo *MI,
                                      const Matrix *m,
-                                     M2_arrayint wts,
                                      gb_array &result_polys)
 {
   const FreeModule *F = m->rows();
@@ -87,7 +65,6 @@ void F4toM2Interface::from_M2_matrix(const VectorArithmetic* VA,
     {
       gbelem *g = new gbelem;
       from_M2_vec(VA, MI, F, m->elem(i), g->f);
-      if (wts != nullptr) poly_set_degrees(VA, MI, wts, g->f, g->deg, g->alpha);
       result_polys.push_back(g);
     }
 }
