@@ -7,7 +7,6 @@
 #include "buffer.hpp"                  // for buffer
 #include "error.h"                     // for error
 #include "f4/f4-m2-interface.hpp"      // for F4toM2Interface
-#include "f4/f4-mem.hpp"               // for F4Mem, F4Vec
 #include "f4/f4-spairs.hpp"            // for F4SPairSet
 #include "f4/f4-types.hpp"             // for row_elem, coefficient_matrix
 #include "f4/hilb-fcn.hpp"             // for HilbertController
@@ -26,11 +25,13 @@
 #include <cstdio>                      // for fprintf, stderr
 #include <algorithm>                   // for stable_sort
 #include <vector>                      // for swap, vector
+#include <atomic>                      // for atomic ints in gauss_reduce
+#include <strstream>
+#include <iostream>
 
 class RingElement;
 
 F4GB::F4GB(const VectorArithmetic* VA,
-           F4Mem *Mem0,
            const MonomialInfo *M0,
            const FreeModule *F0,
            M2_bool collect_syz,
@@ -38,11 +39,11 @@ F4GB::F4GB(const VectorArithmetic* VA,
            M2_arrayint weights0,
            int strategy,
            M2_bool use_max_degree,
-           int max_degree)
+           int max_degree,
+           int numThreads)
     : mVectorArithmetic(VA),
-      M(M0),
-      F(F0),
-      weights(weights0),
+      mMonomialInfo(M0),
+      mFreeModule(F0),
       component_degrees(nullptr),  // need to put this in
       n_pairs_computed(0),
       n_reduction_steps(0),
@@ -52,31 +53,42 @@ F4GB::F4GB(const VectorArithmetic* VA,
       this_degree(),
       is_ideal(F0->rank() == 1),
       hilbert(nullptr),
-      gens(),
-      gb(),
-      lookup(nullptr),
-      S(nullptr),
+      mGenerators(),
+      mGroebnerBasis(),
+      mLookupTable(mMonomialInfo->n_vars()),
       next_col_to_process(0),
       mat(nullptr),
-      H(M0, 17),
-      B(),
+      mMonomialHashTable(M0, 17),
+      mMonomialMemoryBlock(),
       next_monom(),
-      Mem(Mem0),
       clock_sort_columns(0),
       clock_gauss(0),
-      clock_make_matrix(0)
+      mGaussTime(0),
+      mParallelGaussTime(0),
+      mSerialGaussTime(0),
+      mTailReduceTime(0),
+      mNewSPairTime(0),
+      mInsertGBTime(0),
+      clock_make_matrix(0),
+      mNumThreads(mtbb::numThreads(numThreads)),
+#if defined(WITH_TBB)
+      mScheduler(mNumThreads),
+      mSPairSet(mMonomialInfo, mGroebnerBasis, mScheduler)
+#else
+      mSPairSet(mMonomialInfo, mGroebnerBasis)
+#endif
 {
-  lookup = new MonomialLookupTable(M->n_vars());
-  S = new F4SPairSet(M, gb);
+  //  mLookupTable = new MonomialLookupTable(mMonomialInfo->n_vars());
+  //  mSPairSet = new F4SPairSet(mMonomialInfo, mGroebnerBasis);
   mat = new coefficient_matrix;
 
   // TODO: set status?
-  if (M2_gbTrace >= 2) M->show();
+  if (M2_gbTrace >= 3) mMonomialInfo->show();
 }
 
 void F4GB::set_hilbert_function(const RingElement *hf)
 {
-  hilbert = new HilbertController(F, hf); // TODO MES: GC problem?
+  hilbert = new HilbertController(mFreeModule, hf); // TODO MES: GC problem?
 }
 
 void F4GB::delete_gb_array(gb_array &g)
@@ -92,22 +104,41 @@ void F4GB::delete_gb_array(gb_array &g)
 
 F4GB::~F4GB()
 {
-  delete S;
-  delete lookup;
+  //  delete mSPairSet;
+  // delete mLookupTable;
   delete mat;
 
   // Now delete the gens, gb arrays.
-  delete_gb_array(gens);
-  delete_gb_array(gb);
+  delete_gb_array(mGenerators);
+  delete_gb_array(mGroebnerBasis);
+}
+
+void F4GB::poly_set_degrees(const GBF4Polynomial &f,
+                            int &deg_result,
+                            int &alpha) const
+{
+  const monomial_word *w = f.monoms;
+  monomial_word leaddeg = mMonomialInfo->monomial_heft(w);
+  monomial_word deg = leaddeg;
+
+  for (int i = 1; i < f.len; i++)
+    {
+      w = w + mMonomialInfo->monomial_size(w);
+      monomial_word degi = mMonomialInfo->monomial_heft(w);
+      if (degi > deg) deg = degi;
+    }
+  alpha = static_cast<int>(deg - leaddeg);
+  deg_result = static_cast<int>(deg);
 }
 
 void F4GB::new_generators(int lo, int hi)
 {
   for (int i = lo; i <= hi; i++)
     {
-      gbelem *g = gens[i];
+      gbelem *g = mGenerators[i];
+      poly_set_degrees(g->f, g->deg, g->alpha);
       if (g->f.len == 0) continue;
-      S->insert_generator(g->deg, g->f.monoms, i);
+      mSPairSet.insert_generator(g->deg, g->f.monoms, i);
     }
 }
 
@@ -129,11 +160,11 @@ int F4GB::new_column(packed_monomial m)
 int F4GB::find_or_append_column(packed_monomial m)
 {
   packed_monomial new_m;
-  if (H.find_or_insert(m, new_m)) return static_cast<int>(new_m[-1]);
+  if (mMonomialHashTable.find_or_insert(m, new_m)) return static_cast<int>(new_m[-1]);
   // At this point, m is a new monomial to be placed as a column
   m = next_monom;
-  B.intern(1 + M->monomial_size(m));
-  next_monom = B.reserve(1 + M->max_monomial_size());
+  mMonomialMemoryBlock.intern(1 + mMonomialInfo->monomial_size(m));
+  next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
   next_monom++;
   return new_column(m);
 }
@@ -146,34 +177,36 @@ int F4GB::mult_monomials(packed_monomial m, packed_monomial n)
   // If it is there, return its column
   // If not, increment our memory block, and insert a new column.
   packed_monomial new_m;
-  M->unchecked_mult(m, n, next_monom);
-  if (H.find_or_insert(next_monom, new_m))
+  mMonomialInfo->unchecked_mult(m, n, next_monom);
+  if (mMonomialHashTable.find_or_insert(next_monom, new_m))
     return static_cast<int>(
         new_m[-1]);  // monom exists, don't save monomial space
   m = next_monom;
-  B.intern(1 + M->monomial_size(m));
-  next_monom = B.reserve(1 + M->max_monomial_size());
+  mMonomialMemoryBlock.intern(1 + mMonomialInfo->monomial_size(m));
+  next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
   next_monom++;
   return new_column(m);
 }
 
 void F4GB::load_gen(int which)
 {
-  GBF4Polynomial &g = gens[which]->f;
+  GBF4Polynomial &g = mGenerators[which]->f;
 
   row_elem r{};
   r.monom = nullptr;  // This says that this element corresponds to a generator
   r.elem = which;
   r.len = g.len;
-  r.comps = Mem->components.allocate(g.len);
+  // r.comps = Mem->components.allocate(g.len);
+  r.comps = new int[g.len];
   // r.coeffs is already initialized to [nullptr].
 
+  // TODO: this iterator requires knowledge about memory layout of monomials
   monomial_word *w = g.monoms;
   for (int i = 0; i < g.len; i++)
     {
-      M->copy(w, next_monom);
+      mMonomialInfo->copy(w, next_monom);
       r.comps[i] = find_or_append_column(next_monom);
-      w += M->monomial_size(w);
+      w += mMonomialInfo->monomial_size(w);
     }
 
   mat->rows.push_back(r);
@@ -181,20 +214,22 @@ void F4GB::load_gen(int which)
 
 void F4GB::load_row(packed_monomial monom, int which)
 {
-  GBF4Polynomial &g = gb[which]->f;
+  GBF4Polynomial &g = mGroebnerBasis[which]->f;
 
   row_elem r{};
   r.monom = monom;
   r.elem = which;
   r.len = g.len;
-  r.comps = Mem->components.allocate(g.len);
+  // r.comps = Mem->components.allocate(g.len);
+  r.comps = new int[g.len];
   // r.coeffs is already initialized to [nullptr].
-  
+
+  // TODO: this iterator requires knowledge about memory layout of monomials  
   monomial_word *w = g.monoms;
   for (int i = 0; i < g.len; i++)
     {
       r.comps[i] = mult_monomials(monom, w);
-      w += M->monomial_size(w);
+      w += mMonomialInfo->monomial_size(w);
     }
 
   mat->rows.push_back(r);
@@ -211,13 +246,13 @@ void F4GB::process_column(int c)
   column_elem &ce = mat->columns[c];
   if (ce.head >= -1) return;
   int32_t which;
-  bool found = lookup->find_one_divisor_packed(M, ce.monom, which);
+  bool found = mLookupTable.find_one_divisor_packed(mMonomialInfo, ce.monom, which);
   if (found)
     {
       packed_monomial n = next_monom;
-      M->unchecked_divide(ce.monom, gb[which]->f.monoms, n);
-      B.intern(1 + M->monomial_size(n));
-      next_monom = B.reserve(1 + M->max_monomial_size());
+      mMonomialInfo->unchecked_divide(ce.monom, mGroebnerBasis[which]->f.monoms, n);
+      mMonomialMemoryBlock.intern(1 + mMonomialInfo->monomial_size(n));
+      next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
       next_monom++;
       // M->set_component(which, n);
       ce.head = INTSIZE(mat->rows);
@@ -227,21 +262,35 @@ void F4GB::process_column(int c)
     ce.head = -1;
 }
 
+void F4GB::loadSPairRow(spair *p)
+{
+   packed_monomial n = next_monom;
+   mMonomialInfo->unchecked_divide(p->lcm, mGroebnerBasis[p->i]->f.monoms, n);
+   mMonomialMemoryBlock.intern(1 + mMonomialInfo->monomial_size(n));
+   next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
+   next_monom++;
+   load_row(n, p->i);
+}
+
+void F4GB::loadReducerRow(spair *p)
+{
+   packed_monomial n = next_monom;
+   mMonomialInfo->unchecked_divide(p->lcm, mGroebnerBasis[p->j]->f.monoms, n);
+   mMonomialMemoryBlock.intern(1 + mMonomialInfo->monomial_size(n));
+   next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
+   next_monom++;
+   load_row(n, p->j);
+}
+
 void F4GB::process_s_pair(spair *p)
 {
   int c;
 
   switch (p->type)
     {
-      case F4_SPAIR_SPAIR:
+    case SPairType::SPair:
         {
-          packed_monomial n = next_monom;
-          M->unchecked_divide(p->lcm, gb[p->i]->f.monoms, n);
-          B.intern(1 + M->monomial_size(n));
-          next_monom = B.reserve(1 + M->max_monomial_size());
-          next_monom++;
-
-          load_row(n, p->i);
+          loadSPairRow(p);
           c = mat->rows[mat->rows.size() - 1].comps[0];
 
           if (mat->columns[c].head >= -1)
@@ -249,21 +298,15 @@ void F4GB::process_s_pair(spair *p)
           else
             {
               // In this situation, we load the other half as a reducer
-              n = next_monom;
-              M->unchecked_divide(p->lcm, gb[p->j]->f.monoms, n);
-              B.intern(1 + M->monomial_size(n));
-              next_monom = B.reserve(1 + M->max_monomial_size());
-              next_monom++;
-              load_row(n, p->j);
-
+              loadReducerRow(p);
               mat->columns[c].head = INTSIZE(mat->rows) - 1;
             }
           break;
         }
-      case F4_SPAIR_GEN:
+    case SPairType::Generator:
         load_gen(p->i);
         break;
-      default:
+    case SPairType::Retired:
         break;
     }
 }
@@ -284,10 +327,10 @@ void F4GB::reorder_columns()
 
   // sort the columns
 
-  int *column_order = Mem->components.allocate(ncols);
-  int *ord = Mem->components.allocate(ncols);
+  int *column_order = new int[ncols];
+  int *ord = new int[ncols];
 
-  ColumnsSorter C(M, mat);
+  ColumnsSorter C(mMonomialInfo, mat);
 
 // Actual sort of columns /////////////////
 
@@ -301,8 +344,14 @@ void F4GB::reorder_columns()
 
   if (M2_gbTrace >= 2) fprintf(stderr, "ncomparisons = ");
 
+  // std::cout << "-- before sort --" << std::endl;
+  // show_column_info();
+  
   std::stable_sort(column_order, column_order + ncols, C);
 
+  // std::cout << "-- after sort --" << std::endl;  
+  // show_column_info();
+  
   clock_t end_time0 = clock();
   if (M2_gbTrace >= 2) fprintf(stderr, "%ld, ", C.ncomparisons0());
   double nsecs0 = (double)(end_time0 - begin_time0) / CLOCKS_PER_SEC;
@@ -346,8 +395,10 @@ void F4GB::reorder_columns()
     }
 
   std::swap(mat->columns, newcols);
-  Mem->components.deallocate(column_order);
-  Mem->components.deallocate(ord);
+  delete [] column_order;
+  delete [] ord;
+  //Mem->components.deallocate(column_order);
+  //Mem->components.deallocate(ord);
 }
 
 void F4GB::reorder_rows()
@@ -355,7 +406,7 @@ void F4GB::reorder_rows()
   int nrows = INTSIZE(mat->rows);
   int ncols = INTSIZE(mat->columns);
   coefficient_matrix::row_array newrows;
-  VECTOR(long) rowlocs;  // 0..nrows-1 of where row as been placed
+  std::vector<long> rowlocs;  // 0..nrows-1 of where row as been placed
 
   newrows.reserve(nrows);
   rowlocs.reserve(nrows);
@@ -383,7 +434,7 @@ void F4GB::reset_matrix()
 {
   // mat
   next_col_to_process = 0;
-  next_monom = B.reserve(1 + M->max_monomial_size());
+  next_monom = mMonomialMemoryBlock.reserve(1 + mMonomialInfo->max_monomial_size());
   next_monom++;
 }
 
@@ -392,23 +443,22 @@ void F4GB::clear_matrix()
   // Clear the rows first
   for (auto & r : mat->rows)
     {
+      //      std::cout << "at A" << std::endl;
       if (not r.coeffs.isNull())
         mVectorArithmetic->deallocateElementArray(r.coeffs);
-      Mem->components.deallocate(r.comps);
+      // Mem->components.deallocate(r.comps);
+      //      std::cout << "at B" << std::endl;
+      if (r.comps != nullptr)
+        delete [] r.comps;
+      //      std::cout << "at C" << std::endl;
       r.len = 0;
       r.elem = -1;
       r.monom = nullptr;
     }
   mat->rows.clear();
   mat->columns.clear();
-  H.reset();
-  B.reset();
-  if (M2_gbTrace >= 4)
-    {
-      Mem->components.show();
-      Mem->coefficients.show();
-      Mem->show();
-    }
+  mMonomialHashTable.reset();
+  mMonomialMemoryBlock.reset();
 }
 
 void F4GB::make_matrix()
@@ -419,10 +469,13 @@ void F4GB::make_matrix()
      Is this the best order to do it in?  Maybe not...
   */
 
-  spair *p;
-  while ((p = S->get_next_pair()))
+  //spair *p;
+  std::pair<bool,spair> p;
+  p = mSPairSet.get_next_pair();
+  while (p.first)
     {
-      process_s_pair(p);
+      process_s_pair(&p.second);
+      p = mSPairSet.get_next_pair();
     }
 
   while (next_col_to_process < mat->columns.size())
@@ -444,8 +497,89 @@ void F4GB::make_matrix()
   reorder_columns();
 
   // reorder_rows();  // This is only here so we can see what we are doing...?
+
+  // For debugging: let's compute the number of non-zero elements above the diagonal in A matrix.
+  // Also, let's compute the amount of space used for:
+  //  column info
+  //  row info
+  //  sum of values in each row
+  // Maybe so A B C D matrices separately.
+  macaulayMatrixStats();
 }
 
+auto F4GB::macaulayMatrixStats() const -> MacaulayMatrixStats
+{
+  // Want:
+  //  sizes of A, B, C, D.
+  //  number of elements in each part.
+  // loop through the rows, and for each row, determine: in A or C?
+  //   loop through elements in a row: for each, determine: in A/C or B/D
+
+  MacaulayMatrixStats stats;
+
+  // look at 'mat', determine number of rows, cols, etc.
+  long nrows = mat->rows.size();
+  long ncols = mat->columns.size();
+  for (long i=0; i<nrows; ++i)
+    {
+      bool is_pivot = is_pivot_row(i);
+      if (is_pivot)
+        ++stats.mTopAndLeft;
+
+      auto& r = mat->rows[i];
+      for (long j=0; j < r.len; ++j)
+          {
+            auto c = r.comps[j];
+            bool is_left = mat->columns[c].head >= 0;
+            if (is_pivot and is_left) ++stats.mAEntries;
+            if (is_pivot and not is_left) ++stats.mBEntries;
+            if (not is_pivot and is_left) ++stats.mCEntries;
+            if (not is_pivot and not is_left) ++stats.mDEntries;
+          }
+    }
+
+  stats.mAEntries -= stats.mTopAndLeft;
+  stats.mBottom = nrows - stats.mTopAndLeft;
+  stats.mRight = ncols - stats.mTopAndLeft;
+
+  stats.display();
+  return stats;
+}
+
+void F4GB::MacaulayMatrixStats::display(buffer& o)
+{
+  if (M2_gbTrace >= 2)
+    {
+      std::ostringstream s;
+
+      s << "Quad matrix sizes" << std::endl;
+      s << "sizes of quad matrix" << std::endl;
+      s << std::setw(11) << " " << std::setw(11) << mTopAndLeft << std::setw(11) << mRight << std::endl;
+      s << std::setw(11) << mTopAndLeft << std::setw(11) << "A" << std::setw(11) << "B" << std::endl;
+      s << std::setw(11) << mBottom << std::setw(11) << "C" << std::setw(11) << "D" << std::endl;
+      s << std::endl;
+
+      if (M2_gbTrace >= 5)
+        {
+          s << "Quad matrix entries (no diagonal on A)" << std::endl;
+          s << std::setw(11) << " " << std::setw(11) << mTopAndLeft << std::setw(11) << mRight << std::endl;
+          s << std::setw(11) << mTopAndLeft << std::setw(11) << mAEntries << std::setw(11) << mBEntries << std::endl;
+          s << std::setw(11) << mBottom << std::setw(11) << mCEntries << std::setw(11) <<  mDEntries << std::endl;
+          s << std::endl;
+        }
+      
+      o << s.str().c_str();
+    }
+}
+
+void F4GB::MacaulayMatrixStats::display()
+{
+  buffer o;
+  display(o);
+  emit(o.str());
+}
+
+
 ///////////////////////////////////////////////////
 // Gaussian elimination ///////////////////////////
 ///////////////////////////////////////////////////
@@ -457,8 +591,8 @@ const ElementArray& F4GB::get_coeffs_array(row_elem &r)
 
   // At this point, we must go find the coeff array
   if (r.monom == nullptr)  // i.e. a generator
-    return gens[r.elem]->f.coeffs;
-  return gb[r.elem]->f.coeffs;
+    return mGenerators[r.elem]->f.coeffs;
+  return mGroebnerBasis[r.elem]->f.coeffs;
 }
 
 bool F4GB::is_new_GB_row(int row) const
@@ -482,27 +616,137 @@ void F4GB::gauss_reduce(bool diagonalize)
   int ncols = INTSIZE(mat->columns);
 
   int n_newpivots = -1;  // the number of new GB elements in this degree
-  int n_zero_reductions = 0;
+  std::atomic<long> n_zero_reductions = 0;
+
+  mtbb::tick_count t0;
+  mtbb::tick_count t1;
+
   if (hilbert)
     {
       n_newpivots = hilbert->nRemainingExpected();
-      if (n_newpivots == 0) return;
+      if (n_newpivots == 0)
+        {
+          std::cout << "Oh crap, our logic is wrong: should not get here if no new GB elements expected in this degree" << std::endl;
+          return;
+        }
     }
 
-  ElementArray gauss_row { mVectorArithmetic->allocateElementArray(ncols) };
-  for (int i = 0; i < nrows; i++)
+#if defined(WITH_TBB)
+//#if 0
+
+  std::vector<int> spair_rows;
+  for (int i = 0; i < nrows; ++i)
     {
-      row_elem &r = mat->rows[i];
-      if (r.len == 0) continue;  // could happen once we include syzygies...
+      if (not is_pivot_row(i))
+        spair_rows.push_back(i);
+    }
+  
+  std::mutex cout_guard;
+  t0 = mtbb::tick_count::now();
+  using threadLocalDense_t = mtbb::enumerable_thread_specific<ElementArray>;
+  // create a dense array for each thread
+  threadLocalDense_t threadLocalDense([&]() { 
+    return mVectorArithmetic->allocateElementArray(ncols);
+  });
+  
+  if (M2_gbTrace >= 2) {
+    // TODO: where to display this info
+    std::cout << "Number of Threads Used: " << mNumThreads << std::endl; 
+  }
+  
+  mScheduler.execute([&] {
+    mtbb::parallel_for(mtbb::blocked_range<int>{0, INTSIZE(spair_rows)},
+                       [&](const mtbb::blocked_range<int>& r)
+                       {
+                         // long actual_reductions = 0;
+                         //  for (auto i = r.begin(); i != r.end(); ++i)
+                         //    {
+                         //      if (not is_pivot_row(i))
+                         //        ++ actual_reductions;
+                         //    }
+
+                         //  cout_guard.lock();
+                         //  std::cout << "#reductions to do: " << actual_reductions << std::endl;
+                         //  cout_guard.unlock();
+
+                         threadLocalDense_t::reference my_dense = threadLocalDense.local();
+                         for (auto i = r.begin(); i != r.end(); ++i)
+                         {
+                               bool newNonzeroReduction = gauss_reduce_row(spair_rows[i], my_dense);
+                         }
+                       });
+  });
+  for (auto tlDense : threadLocalDense)
+    mVectorArithmetic->deallocateElementArray(tlDense);
+  t1 = mtbb::tick_count::now();
+  mParallelGaussTime += (t1-t0).seconds();
+#endif 
+
+  if (M2_gbTrace >= 2)
+    std::cout << "About to do serial loop, n_newpivots = " << n_newpivots << std::endl;
+  t0 = mtbb::tick_count::now();
+  ElementArray gauss_row { mVectorArithmetic->allocateElementArray(ncols) };
+  for (auto i = 0; i < spair_rows.size(); i++)
+    {
+      auto this_row = spair_rows[i];
+      if ((not hilbert) or (n_newpivots > 0))
+      {
+         bool newNonzeroReduction = gauss_reduce_row(this_row, gauss_row);
+         if (newNonzeroReduction)
+         {
+            row_elem &r = mat->rows[this_row];
+            mVectorArithmetic->makeMonic(r.coeffs);
+            mat->columns[r.comps[0]].head = this_row;
+            if (hilbert) --n_newpivots;
+         }
+         else
+            ++n_zero_reductions;
+      } else if (hilbert)
+        {
+          // Inform code that we don't want a new GB element from this row.
+          // The row will be deleted with clear_matrix.
+          row_elem &r = mat->rows[this_row];
+          r.len = 0;
+        }
+    }
+  mVectorArithmetic->deallocateElementArray(gauss_row);
+  t1 = mtbb::tick_count::now();
+  mSerialGaussTime += (t1-t0).seconds();
+
+  if (M2_gbTrace >= 3)
+    fprintf(stderr, "-- #zeroreductions %ld\n", n_zero_reductions.load());
+
+  t0 = mtbb::tick_count::now();
+  if (diagonalize) tail_reduce();
+  t1 = mtbb::tick_count::now();
+  mTailReduceTime += (t1-t0).seconds();
+}
+
+bool F4GB::is_pivot_row(int index) const
+{
+  row_elem &r = mat->rows[index];
+  int pivotcol = r.comps[0];
+  int pivotrow = mat->columns[pivotcol].head;
+  return (pivotrow == index);
+}
+
+bool F4GB::gauss_reduce_row(int index,
+                            ElementArray& gauss_row)
+{
+   // returns true if the row reduces to a "new" nonzero row
+   // returns false otherwise
+      row_elem &r = mat->rows[index];
+      if (r.len == 0) return false;  // could happen once we include syzygies...
+      if (is_pivot_row(index)) return false;
+
       int pivotcol = r.comps[0];
       int pivotrow = mat->columns[pivotcol].head;
-      if (pivotrow == i) continue;  // this is a pivot row, so leave it alone
 
       const ElementArray& rcoeffs = get_coeffs_array(r);
       n_pairs_computed++;
       mVectorArithmetic->fillDenseArray(gauss_row, rcoeffs, Range(r.comps, r.comps + r.len));
 
-      int firstnonzero = ncols;
+      int firstnonzero = -1;
       int first = r.comps[0];
       int last = r.comps[r.len - 1];
       do
@@ -521,18 +765,23 @@ void F4GB::gauss_reduce(bool diagonalize)
               int last1 = pivot_rowelem.comps[pivot_rowelem.len - 1];
               if (last1 > last) last = last1;
             }
-          else if (firstnonzero == ncols)
+          else if (firstnonzero == -1)
             firstnonzero = first;
           first = mVectorArithmetic->denseNextNonzero(gauss_row, first+1, last);
         }
-      while (first <= last);
+      while (first <= last);  // end do
+
       if (not r.coeffs.isNull())
         mVectorArithmetic->deallocateElementArray(r.coeffs);
-      Mem->components.deallocate(r.comps);
+      //Mem->components.deallocate(r.comps);
+      delete [] r.comps;
+      r.comps = nullptr;
       r.len = 0;
       //Range<int> monomRange;
       //mVectorArithmetic->denseToSparse(gauss_row, r.coeffs, monomRange, firstnonzero, last, mComponentSpace);
-      mVectorArithmetic->denseToSparse(gauss_row, r.coeffs, r.comps, firstnonzero, last, Mem->components);
+      //mVectorArithmetic->denseToSparse(gauss_row, r.coeffs, r.comps, firstnonzero, last, Mem->components);
+      mVectorArithmetic->denseToSparse(gauss_row, r.coeffs, r.comps, firstnonzero, last);
+
       // TODO set r.comps from monomRange.  Question: who has allocated this space??
       // Maybe for now it is just the following line:
       r.len = mVectorArithmetic->size(r.coeffs);
@@ -545,22 +794,9 @@ void F4GB::gauss_reduce(bool diagonalize)
       // the above line leaves gauss_row zero, and also handles the case when
       // r.len is 0 (TODO: check this!!)
       // it also potentially frees the old r.coeffs and r.comps (TODO: ?? r.comps??)
-      if (r.len > 0)
-        {
-          mVectorArithmetic->makeMonic(r.coeffs);
-          mat->columns[r.comps[0]].head = i;
-          if (--n_newpivots == 0) break;
-        }
-      else
-        n_zero_reductions++;
-    }
-  mVectorArithmetic->deallocateElementArray(gauss_row);
-
-  if (M2_gbTrace >= 3)
-    fprintf(stderr, "-- #zeroreductions %d\n", n_zero_reductions);
-
-  if (diagonalize) tail_reduce();
+      return (r.len > 0);
 }
+                            
 
 void F4GB::tail_reduce()
 {
@@ -601,7 +837,8 @@ void F4GB::tail_reduce()
         };
       if (anychange)
         {
-          Mem->components.deallocate(r.comps);
+          //Mem->components.deallocate(r.comps);
+          delete [] r.comps;
           mVectorArithmetic->deallocateElementArray(r.coeffs);
           r.len = 0;
           //Range<int> monomRange;
@@ -610,11 +847,15 @@ void F4GB::tail_reduce()
           //                                       monomRange,
           //                                       firstnonzero, last,
           //                                       mComponentSpace);
+          // mVectorArithmetic->denseToSparse(gauss_row,
+          //                                        r.coeffs,
+          //                                        r.comps,
+          //                                        firstnonzero, last,
+          //                                        Mem->components);
           mVectorArithmetic->denseToSparse(gauss_row,
                                                  r.coeffs,
                                                  r.comps,
-                                                 firstnonzero, last,
-                                                 Mem->components);
+                                                 firstnonzero, last);
           r.len = mVectorArithmetic->size(r.coeffs);
           //r.len = monomRange.size();
           //r.comps = Mem->components.allocate(r.len);
@@ -648,7 +889,7 @@ void F4GB::insert_gb_element(row_elem &r)
   //  insert the monomial into the lookup table
   //  find new pairs associated to this new element
 
-  int nslots = M->max_monomial_size();
+  int nslots = mMonomialInfo->max_monomial_size();
   int nlongs = r.len * nslots;
 
   gbelem *result = new gbelem;
@@ -670,39 +911,50 @@ void F4GB::insert_gb_element(row_elem &r)
     {
       result->f.coeffs.swap(r.coeffs);
     }
-  result->f.monoms = Mem->allocate_monomial_array(nlongs);
+  
+  //result->f.monoms = Mem->allocate_monomial_array(nlongs);
+  result->f.monoms = new monomial_word[nlongs];
 
   monomial_word *nextmonom = result->f.monoms;
   for (int i = 0; i < r.len; i++)
     {
-      M->copy(mat->columns[r.comps[i]].monom, nextmonom);
+      mMonomialInfo->copy(mat->columns[r.comps[i]].monom, nextmonom);
       nextmonom += nslots;
     }
-  Mem->components.deallocate(r.comps);
+  // Mem->components.deallocate(r.comps);
+  delete [] r.comps;
+  r.comps = nullptr;
   r.len = 0;
   result->deg = this_degree;
-  result->alpha = static_cast<int>(M->last_exponent(result->f.monoms));
+  result->alpha = static_cast<int>(mMonomialInfo->last_exponent(result->f.monoms));
   result->minlevel = ELEM_MIN_GB;  // MES: How do
   // we distinguish between ELEM_MIN_GB, ELEM_POSSIBLE_MINGEN?
 
-  int which = INTSIZE(gb);
-  gb.push_back(result);
+  int which = INTSIZE(mGroebnerBasis);
+  mGroebnerBasis.push_back(result);
 
   if (hilbert)
     {
       int x;
-      int *exp = newarray_atomic(int, M->n_vars());
-      M->to_intstar_vector(result->f.monoms, exp, x);
+      //int *exp = newarray_atomic(int, M->n_vars());
+      int *exp = new int[mMonomialInfo->n_vars()];
+      mMonomialInfo->to_intstar_vector(result->f.monoms, exp, x);
       hilbert->addMonomial(exp, x + 1);
-      freemem(exp);
+      delete [] exp;
+      //freemem(exp);
     }
   // now insert the lead monomial into the lookup table
-  varpower_monomial vp = newarray_atomic(varpower_word, 2 * M->n_vars() + 1);
-  M->to_varpower_monomial(result->f.monoms, vp);
-  lookup->insert_minimal_vp(M->get_component(result->f.monoms), vp, which);
-  freemem(vp);
+  //varpower_monomial vp = newarray_atomic(varpower_word, 2 * M->n_vars() + 1);
+  varpower_monomial vp = new varpower_word[2 * mMonomialInfo->n_vars() + 1];
+  mMonomialInfo->to_varpower_monomial(result->f.monoms, vp);
+  mLookupTable.insert_minimal_vp(mMonomialInfo->get_component(result->f.monoms), vp, which);
+  delete [] vp;
+  //freemem(vp);
   // now go forth and find those new pairs
-  S->find_new_pairs(is_ideal);
+  mtbb::tick_count t0 = mtbb::tick_count::now();
+  mSPairSet.find_new_pairs(is_ideal);
+  mtbb::tick_count t1 = mtbb::tick_count::now();
+  mNewSPairTime += (t1-t0).seconds();
 }
 
 void F4GB::new_GB_elements()
@@ -721,6 +973,7 @@ void F4GB::new_GB_elements()
      then
      we don't need to loop through all of these */
 
+  mtbb::tick_count t0 = mtbb::tick_count::now();
   for (int r = 0; r < mat->rows.size(); r++)
     {
       if (is_new_GB_row(r))
@@ -728,6 +981,8 @@ void F4GB::new_GB_elements()
           insert_gb_element(mat->rows[r]);
         }
     }
+  mtbb::tick_count t1 = mtbb::tick_count::now();
+  mInsertGBTime += (t1-t0).seconds();
 }
 
 ///////////////////////////////////////////////////
@@ -738,6 +993,7 @@ void F4GB::do_spairs()
 {
   if (hilbert && hilbert->nRemainingExpected() == 0)
     {
+      mSPairSet.discardSPairsInCurrentDegree();
       if (M2_gbTrace >= 1)
         fprintf(stderr,
                 "-- skipping degree...no elements expected in this degree\n");
@@ -749,7 +1005,7 @@ void F4GB::do_spairs()
   n_lcmdups = 0;
   make_matrix();
 
-  if (M2_gbTrace >= 5)
+  if (M2_gbTrace >= 6)
     {
       fprintf(stderr, "---------\n");
       show_matrix();
@@ -762,7 +1018,13 @@ void F4GB::do_spairs()
   nsecs /= CLOCKS_PER_SEC;
   if (M2_gbTrace >= 2) fprintf(stderr, " make matrix time = %f\n", nsecs);
 
-  if (M2_gbTrace >= 2) H.dump();
+  if (M2_gbTrace >= 3) mMonomialHashTable.dump();
+
+  double oldParallelGauss = mParallelGaussTime;
+  double oldSerialGauss = mSerialGaussTime;
+  double oldTailReduce = mTailReduceTime;
+  double oldNewSPair = mNewSPairTime;
+  double oldInsertNewGB = mInsertGBTime;
 
   begin_time = clock();
   gauss_reduce(true);
@@ -778,9 +1040,12 @@ void F4GB::do_spairs()
   if (M2_gbTrace >= 2)
     {
       fprintf(stderr, " gauss time          = %f\n", nsecs);
+      fprintf(stderr, " parallel gauss time          = %g\n", mParallelGaussTime - oldParallelGauss);
+      fprintf(stderr, " serial gauss time            = %g\n", mSerialGaussTime - oldSerialGauss);
+      fprintf(stderr, " tail reduce time             = %g\n", mTailReduceTime - oldTailReduce);
 
       fprintf(stderr, " lcm dups            = %ld\n", n_lcmdups);
-      if (M2_gbTrace >= 5)
+      if (M2_gbTrace >= 6)
         {
           fprintf(stderr, "---------\n");
           show_matrix();
@@ -788,11 +1053,21 @@ void F4GB::do_spairs()
         }
     }
   new_GB_elements();
-  int ngb = INTSIZE(gb);
+  if (M2_gbTrace >= 2)
+    {
+      fprintf(stderr, " finding new spair time             = %g\n",  mNewSPairTime - oldNewSPair);
+      fprintf(stderr, " number of spairs in queue          = %zu\n", mSPairSet.numberOfSPairs());
+      fprintf(stderr, " insert new gb time                 = %g\n",  mInsertGBTime - oldInsertNewGB - (mNewSPairTime - oldNewSPair));
+    }
+  
+  int ngb = INTSIZE(mGroebnerBasis);
   if (M2_gbTrace >= 1)
     {
       fprintf(stderr, " # GB elements   = %d\n", ngb);
-      if (M2_gbTrace >= 5) show_gb_array(gb);
+      if (M2_gbTrace >= 5) {
+        show_gb_array(mGroebnerBasis);
+        mSPairSet.display();
+      }
     }
 
   clear_matrix();
@@ -801,7 +1076,7 @@ void F4GB::do_spairs()
 enum ComputationStatusCode F4GB::computation_is_complete(StopConditions &stop_)
 {
   // This handles everything but stop_.always, stop_.degree_limit
-  if (stop_.basis_element_limit > 0 && gb.size() >= stop_.basis_element_limit)
+  if (stop_.basis_element_limit > 0 && mGroebnerBasis.size() >= stop_.basis_element_limit)
     return COMP_DONE_GB_LIMIT;
   if (stop_.pair_limit > 0 && n_pairs_computed >= stop_.pair_limit)
     return COMP_DONE_PAIR_LIMIT;
@@ -826,13 +1101,13 @@ void F4GB::test_spair_code()
   // gb matrix, and then calling find_new_pairs after each one.
   // It displays the list of spairs after each generator.
 
-  gb.push_back(gens[0]);
-  for (int i = 1; i < gens.size(); i++)
+  mGroebnerBasis.push_back(mGenerators[0]);
+  for (int i = 1; i < mGenerators.size(); i++)
     {
-      gb.push_back(gens[i]);
-      S->find_new_pairs(false);
+      mGroebnerBasis.push_back(mGenerators[i]);
+      mSPairSet.find_new_pairs(false);
       fprintf(stderr, "---Just inserted element %d---\n", i);
-      S->display();
+      mSPairSet.display();
     }
 }
 
@@ -841,7 +1116,7 @@ enum ComputationStatusCode F4GB::start_computation(StopConditions &stop_)
   clock_sort_columns = 0;
   clock_gauss = 0;
   clock_make_matrix = 0;
-  int npairs;
+  int npairs = 0;
 
   //  test_spair_code();
 
@@ -858,13 +1133,17 @@ enum ComputationStatusCode F4GB::start_computation(StopConditions &stop_)
       is_done = computation_is_complete(stop_);
       if (is_done != COMP_COMPUTING) break;
 
-      this_degree = S->prepare_next_degree(-1, npairs);
-
-      if (npairs == 0)
+      //this_degree = mSPairSet.prepare_next_degree(-1, npairs);
+      auto thisDegInfo = mSPairSet.setThisDegree();
+      
+      if (not thisDegInfo.first)
         {
           is_done = COMP_DONE;
           break;
         }
+
+      this_degree = thisDegInfo.second;
+      
       if (stop_.stop_after_degree && this_degree > stop_.degree_limit->array[0])
         {
           is_done = COMP_DONE_DEGREE_LIMIT;
@@ -914,18 +1193,39 @@ enum ComputationStatusCode F4GB::start_computation(StopConditions &stop_)
       fprintf(stderr,
               "total time for gauss: %f\n",
               ((double)clock_gauss) / CLOCKS_PER_SEC);
+      fprintf(stderr,
+              "parallel tbb time for gauss: %g\n",
+              mParallelGaussTime);
+      fprintf(stderr,
+              "serial tbb time for gauss: %g\n",
+              mSerialGaussTime);
+      fprintf(stderr,
+              "total time for finding new spairs: %g\n",
+              mNewSPairTime);
+      fprintf(stderr,
+              "total time for creating pre spairs: %g\n",
+              mSPairSet.secondsToCreatePrePairs());
+      fprintf(stderr,
+              "total time for minimizing new spairs: %g\n",
+              mSPairSet.secondsToMinimizePairs());
+      fprintf(stderr,
+              "total time for inserting new gb elements: %g\n",
+              mInsertGBTime);
       fprintf(stderr,
               "number of spairs computed           : %ld\n",
               n_pairs_computed);
       fprintf(stderr,
               "number of reduction steps           : %ld\n",
               n_reduction_steps);
+      if (M2_gbTrace >= 3)
+      {
+        fprintf(stderr,
+              "number of spairs removed by criterion = %ld\n",
+              mSPairSet.n_unneeded_pairs());
+        mMonomialInfo->show();
+      }
     }
 
-  fprintf(stderr,
-          "number of spairs removed by criterion = %ld\n",
-          S->n_unneeded_pairs());
-  M->show();
   return is_done;
 }
 
@@ -942,10 +1242,10 @@ void F4GB::show_gb_array(const gb_array &g) const
   for (int i = 0; i < g.size(); i++)
     {
       vec v = F4toM2Interface::to_M2_vec(
-          mVectorArithmetic, M, g[i]->f, F);
+          mVectorArithmetic, mMonomialInfo, g[i]->f, mFreeModule);
       o << "element " << i << " degree " << g[i]->deg << " alpha "
         << g[i]->alpha << newline << "    ";
-      F->get_ring()->vec_text_out(o, v);
+      mFreeModule->get_ring()->vec_text_out(o, v);
       o << newline;
     }
   emit(o.str());
@@ -960,18 +1260,19 @@ void F4GB::show_row_info() const
       if (mat->rows[i].monom == nullptr)
         fprintf(stderr, "generator");
       else
-        M->show(mat->rows[i].monom);
+        mMonomialInfo->show(mat->rows[i].monom);
       fprintf(stderr, "\n");
     }
 }
 
 void F4GB::show_column_info() const
 {
+  mMonomialInfo->show();
   // Debugging routine
   for (int i = 0; i < mat->columns.size(); i++)
     {
       fprintf(stderr, "head %4d monomial ", mat->columns[i].head);
-      M->show(mat->columns[i].monom);
+      mMonomialInfo->show(mat->columns[i].monom);
       fprintf(stderr, "\n");
     }
 }
@@ -980,7 +1281,7 @@ void F4GB::show_matrix()
 {
   // Debugging routine
   MutableMatrix *q = F4toM2Interface::to_M2_MutableMatrix(
-      mVectorArithmetic, mat, gens, gb);
+      mVectorArithmetic, mat, mGenerators, mGroebnerBasis);
   buffer o;
   q->text_out(o);
   emit(o.str());
