@@ -133,6 +133,10 @@ const Matrix /* or null */ *IM2_Matrix_random(
     int special_type, /* 0: general, 1:upper triangular, others? */
     int preference);  /* connected to rawRandomConstantMatrix, OK */
 
+const Matrix* /* or null */ rawMatrixReadMsolveString(const Ring* R, M2_string contents);
+
+const Matrix* /* or null */ rawMatrixReadMsolveFile(const Ring* R, M2_string filename);
+
 /**********************************************************************************/
 
 M2_bool IM2_Matrix_is_zero(const Matrix *M); /* drg: connected rawIsZero*/
