@@ -1,5 +1,5 @@
 ;;; M2.el --- Major mode for editing Macaulay2 source core -*- lexical-binding: t -*-
-;; Version: 1.24.11
+;; Version: 1.25.05
 ;; Keywords: languages Macaulay2
 ;; URL: https://github.com/Macaulay2/M2-emacs
 ;; Package-Requires: ((emacs "24.1"))
@@ -45,6 +45,40 @@
 ;; M2-comint-mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(defvar M2-error-regexp-alist
+  '(
+    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+    ;; printMessage (stdiop.d) ;;
+    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+    ;; error messages, e.g.,
+    ;; i1 : load "packages/Macaulay2Doc/demo1.m2"; g 2
+    ;; packages/Macaulay2Doc/demo1.m2:8:12:(3):[2]: error: division by zero
+    ;;  (1                                                           1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
+     1 2 3)
+    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+    ;; net(FilePosition) (debugging.m2) ;;
+    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+    ;; start & end line/column numbers, e.g.,:
+    ;; i1 : locate (rank, Matrix)
+    ;; o1 = m2/matrix1.m2:663:19-666:20
+    ;;  (1                                                           1)   (2      2)   (3      3)   (4      4)   (5      5)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
+     1 (2 . 4) (3 . 5) 0)
+    ;; no end line/column numbers, e.g.,:
+    ;; i2 : locate makeDocumentTag rank
+    ;; o2 = ../Macaulay2Doc/functions/rank-doc.m2:34:0
+    ;;  (1                                                          1)   (2      2)   (3      3)
+    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)"
+     1 2 3 0))
+  "Regular expressions for matching file positions in Macaulay2 output.")
+
+(defvar M2-transform-file-match-alist
+  '(("^stdio$" nil)
+    ("^currentString$" nil)
+    ("^[0-9][0-9]$" nil))
+  "List of filenames not to match in Macaulay2 output.")
+
 ;;;###autoload
 (define-derived-mode M2-comint-mode comint-mode "Macaulay2 Interaction"
   "Major mode for interacting with a Macaulay2 process.\n\n\\{M2-comint-mode-map}"
@@ -52,6 +86,7 @@
   (setq comint-prompt-regexp M2-comint-prompt-regexp)
   (add-hook 'comint-input-filter-functions #'M2-comint-forget-errors nil t)
   (add-hook 'comint-preoutput-filter-functions 'M2-info-help nil t)
+  (add-hook 'comint-output-filter-functions #'M2-comint-fix-unclosed nil t)
   (setq-local compilation-error-regexp-alist M2-error-regexp-alist)
   (setq-local compilation-transform-file-match-alist
 	      M2-transform-file-match-alist)
@@ -126,12 +161,13 @@
 
 ;; menus
 
-(setq M2-common-menu
+(defvar M2-common-menu
       '(["Match previous bracketed input" M2-match-previous-bracketed-input]
 	["Match next bracketed input"     M2-match-next-bracketed-input]
 	["Set demo buffer"                M2-set-demo-buffer]
 	["Switch to demo buffer"          M2-switch-to-demo-buffer]
-	["Start demo"                     M2-demo]))
+	["Start demo"                     M2-demo])
+      "Common parts of menus for both `M2-mode' and `M2-comint-mode'.")
 
 (easy-menu-define M2-menu M2-mode-map
   "Menu for Macaulay2 major mode"
@@ -346,40 +382,6 @@ can be executed with \\[M2-send-to-program]."
      (re-search-backward "<<<")
      (match-end 0))))
 
-(defvar M2-error-regexp-alist
-  '(
-    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-    ;; printMessage (stdiop.d) ;;
-    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-    ;; error messages, e.g.,
-    ;; i1 : load "packages/Macaulay2Doc/demo1.m2"; g 2
-    ;; packages/Macaulay2Doc/demo1.m2:8:12:(3):[2]: error: division by zero
-    ;;  (1                                                           1)   (2      2)   (3      3)
-    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\):([0-9]+):\\[[0-9]+\\]"
-     1 2 3)
-    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-    ;; net(FilePosition) (debugging.m2) ;;
-    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-    ;; start & end line/column numbers, e.g.,:
-    ;; i1 : locate (rank, Matrix)
-    ;; o1 = m2/matrix1.m2:663:19-666:20
-    ;;  (1                                                           1)   (2      2)   (3      3)   (4      4)   (5      5)
-    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\):\\([0-9]+\\)"
-     1 (2 . 4) (3 . 5) 0)
-    ;; no end line/column numbers, e.g.,:
-    ;; i2 : locate makeDocumentTag rank
-    ;; o2 = ../Macaulay2Doc/functions/rank-doc.m2:34:0
-    ;;  (1                                                          1)   (2      2)   (3      3)
-    ("\\(?:\\(?1:[[:alnum:]/._][[:alnum:]/._-]*\\)\\|\"\\(?1:.+\\)\"\\):\\([0-9]+\\):\\([0-9]+\\)"
-     1 2 3 0))
-  "Regular expressions for matching file positions in Macaulay2 output.")
-
-(defvar M2-transform-file-match-alist
-  '(("^stdio$" nil)
-    ("^currentString$" nil)
-    ("^[0-9][0-9]$" nil))
-  "List of filenames not to match in Macaulay2 output.")
-
 (define-obsolete-function-alias
   'M2-send-input 'comint-send-input "1.23")
 
@@ -465,6 +467,13 @@ for more."
     (M2--send-to-program-helper send-to-buffer start end))
   (forward-paragraph))
 
+(defvar M2-demo-buffer
+  (with-current-buffer (get-buffer-create "*M2-demo-buffer*")
+    (M2-mode)
+    (current-buffer))
+  "The buffer from which lines are obtained by `M2-get-input-from-demo-buffer'.
+Set it with `M2-set-demo-buffer'." )
+
 (defun M2-set-demo-buffer()
   "Set the variable M2-demo-buffer to the current buffer, so that later,
 `M2-get-input-from-demo-buffer' can obtain lines from this buffer."
@@ -476,6 +485,8 @@ for more."
   (interactive)
   (switch-to-buffer M2-demo-buffer))
 
+(declare-function toggle-scroll-bar "scroll-bar")
+
 (defun M2-demo()
   "Sets up a new frame with a big font for a Macaulay2 demo."
   (interactive)
@@ -527,6 +538,26 @@ Otherwise, send the input to Macaulay2."
 	(substring string end))
     string))
 
+(defun M2-comint-insert-invisible-at-bol (string)
+  "Insert STRING with the invisible property at the beginning of the line."
+  (save-excursion
+    (beginning-of-line)
+    (insert string)
+    (put-text-property (- (point) (length string)) (point) 'invisible t)))
+
+(defun M2-comint-fix-unclosed (string)
+  "Close any unclosed strings or comments from the output.
+STRING is the current Macaulay2 output, which we check to see whether we're at
+a new input prompt."
+  (ignore string)
+  (when (string-match-p "^[ \t]*i+[1-9][0-9]* : " string)
+    (let ((syntax (syntax-ppss (point))))
+      (cond
+       ((nth 3 syntax) (M2-comint-insert-invisible-at-bol "\""))
+       ((nth 4 syntax) (M2-comint-insert-invisible-at-bol "*-"))))))
+
+(declare-function compilation-forget-errors "compile")
+
 (defun M2-comint-forget-errors (string)
   "Run `compilation-forget-errors' to flush compilation mode's cache.
 Otherwise, jumping to source will go to the wrong location when a file has
@@ -614,13 +645,6 @@ line based on the depth of the parentheses in the code."
 	    (M2-this-line-indent-amount))
      (delete-horizontal-space))))
 
-(defvar M2-demo-buffer
-  (with-current-buffer (get-buffer-create "*M2-demo-buffer*")
-    (M2-mode)
-    (current-buffer))
-  "The buffer from which lines are obtained by `M2-get-input-from-demo-buffer'.
-Set it with `M2-set-demo-buffer'." )
-
 ;;; "blink" evaluated region (heavily inspired by ESS)
 
 (defcustom M2-blink-region-flag t
