@@ -410,14 +410,14 @@ function gfxReorder(el) {
     }
 }
 
-function gNode(nd) {
+function gNode(nd,vec) {
     if (nd instanceof HTMLCollection) { // we're in trouble -- id used multiple times
 	i=0;
 	while (i<nd.length && !currentEl.contains(nd[i])) i++;
 	if (i<nd.length) nd=nd[i]; else return; // shouldn't happen
     }
     return function(cmat) {
-	return nd.gfxdata.cmatrix.vectmultiply([0,0,0,1]); // TODO optimize
+	return nd.gfxdata.cmatrix.vectmultiply(vec?vec:[0,0,0,1]); // TODO optimize
     }
 }
 
